// CBNapplet

import java.applet.Applet;
import java.awt.*;

public class CBNapplet extends Applet {
	private static final String APPLET_INFO = "CBNapplet\n(c) 1998, Michael R. Miller\n";
	
	private ControlPanel controlPanel;
	private Algorithm algorithm;
	private Viewer viewer;

	public String getAppletInfo() {
		return APPLET_INFO;
	}
	
	public void init() {
		setLayout (new GridLayout (1, 2, 5, 5));

		if (controlPanel == null)
			controlPanel = new ControlPanel (this);
		
		int bkgd = 0xFFFFFF;
		try {
			bkgd = Integer.parseInt (getParameter ("background color"), 16);
		} catch (NumberFormatException nfe) { }
		setBackground (new Color (bkgd));
	}
	
	public void start() {
		reload();
	}
	
	public void stop() {
		if (algorithm != null) algorithm.requestStop();
		if (viewer != null) viewer.freeResources();
	}
	
	public void destroy() {
		if (algorithm != null) algorithm.requestStop();
		if (viewer != null) viewer.freeResources();
	}

	public void reload() {
		controlPanel.clearParameters();
		int algorithmID = controlPanel.getSelectedAlgorithm();
		
		if (algorithm != null) algorithm.requestStop();
		algorithm = AlgorithmFactory.createAlgorithm (algorithmID, this);
		algorithm.addParametersToControlPanel();

		if (viewer != null) viewer.freeResources();
		viewer = ViewerFactory.createViewer (algorithmID, this);

		removeAll();
		Panel p = new Panel();
		p.add (viewer);		
		add (p);		
		add (controlPanel);
		
		Thread thr = new Thread (algorithm);
		thr.setPriority (Thread.NORM_PRIORITY-1);
		thr.start();
		
		validate();
	}
	
	public void restart() {
		int algorithmID = controlPanel.getSelectedAlgorithm();
		
		if (algorithm != null) algorithm.requestStop();
 		algorithm = AlgorithmFactory.createAlgorithm (algorithmID, this);
		
		Thread thr = new Thread (algorithm);
		thr.setPriority (Thread.NORM_PRIORITY-1);
		thr.start();
	}
	
	public ControlPanel getControlPanel() {
		return controlPanel;
	}
	
	public Viewer getViewer() {
		return viewer;
	}
}
